//*****************************************************************************
// File Name	: i2c7seg.c
// 
// Title		: I2C -> 7SEG LED slave
// Revision		: 1.0
// Notes		: 
// Target MCU	: Atmel AVR series ATtiny2313
// Tool Chain	: WinAVR-20081205
// 
// Revision History:
// When			Who			Description of change
// -----------	-----------	-----------------------
// 14-Mar-2009	S.Takagi	Created the program
//*****************************************************************************

//----- Include Files ---------------------------------------------------------
#include <inttypes.h>
#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

//#include "usiTwiSlave.h"
static const uint8_t TWI_slaveAddress = 0x33;

#define ESC (0x1B)
#define CHARLIE_1_D	PD0
#define CHARLIE_2_D	PD1
#define CHARLIE_3_D	PD2
#define CHARLIE_4_D	PD3
#define CHARLIE_5_D	PD4
#define CHARLIE_6_D	PD5
#define CHARLIE_7_D	PD6
#define CHARLIE_8_B	PB0
#define CHARLIE_9_B	PB1

#define LED_dp	7
#define LED_g	6
#define LED_f	5
#define LED_e	4
#define LED_d	3
#define LED_c	2
#define LED_b	1
#define LED_a	0

volatile uint8_t digit[9];
static uint8_t NumberOfDigits = 9;

unsigned char __attribute__ ((progmem)) LedChar[] = {
	0x3F, 0x06, 0x5b, 0x4F, 0x66, 0x6D, 0x7D, 0x07, 	// 01234567
	0x7F, 0x6F, 0x77, 0x7C, 0x39, 0x5E, 0x79, 0x71, 	// 89AbCdEF
	0x00, 0x40, 0x48, 0x49, 0x08, 0x01, 0x41, 0x09, 	//  -=_~=
	0x5C, 0x63, 0x1C, 0x62, 0x58, 0x36, 0x7B, 0x76  	// cIIeH
};


ISR(TIMER0_OVF_vect)
{
	static uint8_t current_digit = 0;
	uint8_t segment;

	segment = digit[current_digit];

	// set ports direction to 'input'
	DDRD &= ~(
				(1<<CHARLIE_1_D) |
				(1<<CHARLIE_2_D) |
				(1<<CHARLIE_3_D) |
				(1<<CHARLIE_4_D) |
				(1<<CHARLIE_5_D) |
				(1<<CHARLIE_6_D) |
				(1<<CHARLIE_7_D)
			);

	DDRB &= ~(
				(1<<CHARLIE_8_B) |
				(1<<CHARLIE_9_B)
			);

	// set ports value to 'LOW'
	PORTD &= ~(
				(1<<CHARLIE_1_D) |
				(1<<CHARLIE_2_D) |
				(1<<CHARLIE_3_D) |
				(1<<CHARLIE_4_D) |
				(1<<CHARLIE_5_D) |
				(1<<CHARLIE_6_D) |
				(1<<CHARLIE_7_D)
			);

	PORTB &= ~(
				(1<<CHARLIE_8_B) |
				(1<<CHARLIE_9_B)
			);

	// set corresponding port to 'output' to sink
	if (segment & 0b00000001) DDRD |= (1<<CHARLIE_1_D);
	if (segment & 0b00000010) DDRD |= (1<<CHARLIE_2_D);
	if (segment & 0b00000100) DDRD |= (1<<CHARLIE_3_D);
	if (segment & 0b00001000) DDRD |= (1<<CHARLIE_4_D);
	if (segment & 0b00010000) DDRD |= (1<<CHARLIE_5_D);
	if (segment & 0b00100000) DDRD |= (1<<CHARLIE_6_D);
	if (segment & 0b01000000) DDRD |= (1<<CHARLIE_7_D);
	if (segment & 0b10000000) DDRB |= (1<<CHARLIE_8_B);

	// bit-fiddling
	switch (current_digit) {
	case 0:
		if (segment & 0b00000001) DDRB |= (1<<CHARLIE_9_B);
		DDRD  |= (1<<CHARLIE_1_D);
		PORTD |= (1<<CHARLIE_1_D);
		break;
	case 1:
		if (segment & 0b00000010) DDRB |= (1<<CHARLIE_9_B);
		DDRD  |= (1<<CHARLIE_2_D);
		PORTD |= (1<<CHARLIE_2_D);
		break;
	case 2:
		if (segment & 0b00000100) DDRB |= (1<<CHARLIE_9_B);
		DDRD  |= (1<<CHARLIE_3_D);
		PORTD |= (1<<CHARLIE_3_D);
		break;
	case 3:
		if (segment & 0b00001000) DDRB |= (1<<CHARLIE_9_B);
		DDRD  |= (1<<CHARLIE_4_D);
		PORTD |= (1<<CHARLIE_4_D);
		break;
	case 4:
		if (segment & 0b00010000) DDRB |= (1<<CHARLIE_9_B);
		DDRD  |= (1<<CHARLIE_5_D);
		PORTD |= (1<<CHARLIE_5_D);
		break;
	case 5:
		if (segment & 0b00100000) DDRB |= (1<<CHARLIE_9_B);
		DDRD  |= (1<<CHARLIE_6_D);
		PORTD |= (1<<CHARLIE_6_D);
		break;
	case 6:
		if (segment & 0b01000000) DDRB |= (1<<CHARLIE_9_B);
		DDRD  |= (1<<CHARLIE_7_D);
		PORTD |= (1<<CHARLIE_7_D);
		break;
	case 7:
		if (segment & 0b10000000) DDRB |= (1<<CHARLIE_9_B);
		DDRB  |= (1<<CHARLIE_8_B);
		PORTB |= (1<<CHARLIE_8_B);
		break;
	case 8:
		DDRB  |= (1<<CHARLIE_9_B);
		PORTB |= (1<<CHARLIE_9_B);
		break;
	}


	if(++current_digit > NumberOfDigits)
		current_digit = 0;

}

int main()
{
	unsigned char i, j;

	TCCR0B = (1<<CS00)|(1<<CS01);
	TIMSK |= (1<<TOIE0);
	TCNT0 = 0xFF;

	sei();

	while(1) {

		for (i = 0; i < 8; i++) {
			for (j = 0; j < NumberOfDigits; j++) {
				digit[j] = 1<<i;
				_delay_ms(100);
				digit[j] = 0;
			}
		}

		digit[8] = pgm_read_byte(LedChar+0);
		digit[7] = pgm_read_byte(LedChar+1);
		digit[6] = pgm_read_byte(LedChar+2);
		digit[5] = pgm_read_byte(LedChar+3);
		digit[4] = pgm_read_byte(LedChar+4);
		digit[3] = pgm_read_byte(LedChar+5);
		digit[2] = pgm_read_byte(LedChar+6);
		digit[1] = pgm_read_byte(LedChar+7);
		digit[0] = pgm_read_byte(LedChar+8);

		_delay_ms(3000);

		digit[0] |= (1<<LED_dp);
		digit[1] |= (1<<LED_dp);
		digit[2] |= (1<<LED_dp);
		digit[3] |= (1<<LED_dp);
		digit[4] |= (1<<LED_dp);
		digit[5] |= (1<<LED_dp);
		digit[6] |= (1<<LED_dp);
		digit[7] |= (1<<LED_dp);
		digit[8] |= (1<<LED_dp);

		_delay_ms(3000);

		digit[0] = 0x00;
		digit[1] = 0x00;
		digit[2] = 0x00;
		digit[3] = 0x00;
		digit[4] = 0x00;
		digit[5] = 0x00;
		digit[6] = 0x00;
		digit[7] = 0x00;
		digit[8] = 0x00;

	}

	return 0;
}
